# Generated by roxygen2: do not edit by hand

S3method("$",httr2_headers)
S3method("[",httr2_headers)
S3method("[[",httr2_headers)
S3method(close,httr2_response)
S3method(format,httr2_redacted_sentinel)
S3method(print,httr2_cmd)
S3method(print,httr2_headers)
S3method(print,httr2_json)
S3method(print,httr2_oauth_client)
S3method(print,httr2_obfuscated)
S3method(print,httr2_redacted_sentinel)
S3method(print,httr2_request)
S3method(print,httr2_response)
S3method(print,httr2_token)
S3method(print,httr2_url)
S3method(str,httr2_headers)
S3method(str,httr2_obfuscated)
S3method(str,httr2_redacted_sentinel)
export("%>%")
export(StreamingBody)
export(curl_help)
export(curl_translate)
export(example_github_client)
export(example_url)
export(is_online)
export(iterate_with_cursor)
export(iterate_with_link_url)
export(iterate_with_offset)
export(jwt_claim)
export(jwt_encode_hmac)
export(jwt_encode_sig)
export(last_request)
export(last_request_json)
export(last_response)
export(last_response_json)
export(local_mocked_responses)
export(local_verbosity)
export(new_response)
export(oauth_cache_clear)
export(oauth_cache_path)
export(oauth_client)
export(oauth_client_req_auth)
export(oauth_client_req_auth_body)
export(oauth_client_req_auth_header)
export(oauth_client_req_auth_jwt_sig)
export(oauth_flow_auth_code)
export(oauth_flow_auth_code_listen)
export(oauth_flow_auth_code_parse)
export(oauth_flow_auth_code_pkce)
export(oauth_flow_auth_code_url)
export(oauth_flow_bearer_jwt)
export(oauth_flow_client_credentials)
export(oauth_flow_device)
export(oauth_flow_password)
export(oauth_flow_refresh)
export(oauth_flow_token_exchange)
export(oauth_redirect_uri)
export(oauth_token)
export(oauth_token_cached)
export(obfuscate)
export(obfuscated)
export(req_auth_aws_v4)
export(req_auth_basic)
export(req_auth_bearer_token)
export(req_body_file)
export(req_body_form)
export(req_body_json)
export(req_body_json_modify)
export(req_body_multipart)
export(req_body_raw)
export(req_cache)
export(req_cookie_preserve)
export(req_cookies_set)
export(req_dry_run)
export(req_error)
export(req_get_body)
export(req_get_body_type)
export(req_get_headers)
export(req_get_method)
export(req_get_url)
export(req_headers)
export(req_headers_redacted)
export(req_method)
export(req_oauth)
export(req_oauth_auth_code)
export(req_oauth_bearer_jwt)
export(req_oauth_client_credentials)
export(req_oauth_device)
export(req_oauth_password)
export(req_oauth_refresh)
export(req_oauth_token_exchange)
export(req_options)
export(req_perform)
export(req_perform_connection)
export(req_perform_iterative)
export(req_perform_parallel)
export(req_perform_promise)
export(req_perform_sequential)
export(req_perform_stream)
export(req_progress)
export(req_proxy)
export(req_retry)
export(req_template)
export(req_throttle)
export(req_timeout)
export(req_url)
export(req_url_path)
export(req_url_path_append)
export(req_url_query)
export(req_url_relative)
export(req_user_agent)
export(req_verbose)
export(request)
export(resp_body_html)
export(resp_body_json)
export(resp_body_raw)
export(resp_body_string)
export(resp_body_xml)
export(resp_check_content_type)
export(resp_check_status)
export(resp_content_type)
export(resp_date)
export(resp_encoding)
export(resp_has_body)
export(resp_header)
export(resp_header_exists)
export(resp_headers)
export(resp_is_error)
export(resp_link_url)
export(resp_raw)
export(resp_request)
export(resp_retry_after)
export(resp_status)
export(resp_status_desc)
export(resp_stream_aws)
export(resp_stream_is_complete)
export(resp_stream_lines)
export(resp_stream_raw)
export(resp_stream_sse)
export(resp_timing)
export(resp_url)
export(resp_url_path)
export(resp_url_queries)
export(resp_url_query)
export(response)
export(response_json)
export(resps_data)
export(resps_failures)
export(resps_ok)
export(resps_requests)
export(resps_successes)
export(secret_decrypt)
export(secret_decrypt_file)
export(secret_encrypt)
export(secret_encrypt_file)
export(secret_has_key)
export(secret_make_key)
export(secret_read_rds)
export(secret_write_rds)
export(signal_total_pages)
export(throttle_status)
export(url_build)
export(url_modify)
export(url_modify_query)
export(url_modify_relative)
export(url_parse)
export(url_query_build)
export(url_query_parse)
export(with_mocked_responses)
export(with_verbosity)
import(R6)
import(rlang)
importFrom(glue,glue)
importFrom(lifecycle,deprecated)
importFrom(magrittr,"%>%")
