## R CMD check results

0 errors | 0 warnings | 0 notes

## revdepcheck results

We checked 57 reverse dependencies, comparing R CMD check results across CRAN and dev versions of this package.

 * We saw 1 new problems with happign: this looks like user error,
   but I've supplied a patch at https://github.com/paul-carteron/happign/pull/22.
 * We failed to check 0 packages
