# resp_stream_lines(warn) is deprecated unless FALSE

    Code
      . <- resp_stream_lines(resp, warn = TRUE)
    Condition
      Warning:
      The `warn` argument of `resp_stream_lines()` is deprecated as of httr2 1.2.3.

